;
;    Source files for the project
;	drivera.asm	The main file
;	long.inc	32 bit header file				       
;								       
;    Schematic overview:					       
;	The design has 4 motor contollers.  There are two per axis       
;	with each axis packed into one 8 bit word.  Access is via      
;	register B and an 8 bit latch per axis.  Selects for the	       
;	latches are in register C.				       
;
;	A pot between +12 and ground is tied to an analog in and can   
;	be used to monitor the battery voltage. Two limit switches      
;	are implemented to allow the RA movement to be monitored.      
;
;	The internal RS232 module is expected to be used in async      
;	mode and is routed to a MAX222.				       
;
;    General Strategy:						       
;	The software implements a command loop that monitors the       
;	RS232 port for characters in polled mode.
;	The port is set up at 9600 baud, 8 bits, no parity
;	Characters are echoed and buffered to a full line terminated by a CR.
;	The line is parsed and executed.  All motor commands are kicked    
;	off but the timing is handled in interrupt commnands.	       
;	Given the pain of doing math on this system, the acceleration  
;	and speeds are currently compiled in.
;
;    Memory layout:
;	The first 0x800 is all interrupt code and interrupt support routines.
;	The second 0x800 is all user code and user level support routines.
;	A slow, goofy macro package is used to implement a user level stack
;	in the 2nd bank of registers.  The first bank is used for interrupt
;	processing.  The user level code is restricted to one deep stack
;	usage to allow the max. speed at interrupt level.
;
;    Timers:
;	Timer 1 is set to be free-running, clocked at 1/4 FOSC = 1MHZ
;	CCPR1 is used to time RA events 		       
;	CCPR2 is used to time Dec events
;	The timer is never shut off; match interrupts are turned on and off
;	as the drives need attention
;
;	Timer 0 is used as a free running clock, clocked at 1MHZ.  The prescaler is
;	set to 256 so that the timer rolls over every 65.536 MS or a little more
;	than 15 times per second.  It is used for gross user level timing of events
;	such as polling the battery value.
;
;    Directions
;	+ advances in terms of RA or Dec, ie, East or North
;	- decrements RA or DEC, ie, West or South
;	Tracking is inherently movement to the West and so is negative.
;
;    Command set						       
;	F<m><dir> <count>	Full step <count> steps		       
;	GC			Gets the count for both axes
;	GB			Gets the atod value and the rolling average value    
;	M<m><dir>		Starts an axis slewing       
;	Q<m>			Stop
;	I<m>			Idle: Quit and turn power off				       
;	R			Reset				       
;	SC<m> <count>		Sets the saved count for an axis
;	SS<speed>		Sets speed for next Slew command (both axes)
;	T			Starts RA tracking
;	U<m><dir> <count>	Microstep <count> steps		       
;
;	Abbreviations:						       
;	<count> - signed 32 bit count of full steps		       
;	<m> can be "R" or "D" and specifies which motor to use	       
;	<speed> is "T" for Tracking speed
;		   "G" for Guide speed
;		   "L" for Lunar
;		   "S" for Solar
;		   "M" for Medium
;		   "F" for Fast
;	<dir> is "+" or "-"
		page
; Main assembly options                           
		list	p=16c73b	; list directive to define processor
		radix   dec
		#include <p16c73b.inc>	; processor specific variable definitions
		nolist
		#include "long.inc" 	; Long (32 bit) arithmetic package header
		__CONFIG	_CP_OFF & _WDT_OFF & _BODEN_ON & _PWRTE_ON & _XT_OSC
;
; Macros
;
; User level subroutine call 
; 	Semantically same as call but takes more bytes and trashes z in status
;	Uses a stack starting in bank 2 at the top
;	Dest is the address of the routine to call
;
mi		set	0
mcall		macro	dest		;
mi		set	mi+1		;
		movwf	b0mwsave	; Stash w in bank 0 save
		movlw	high mcall#v(mi)
		movwf	PCLATH		; Means sub must be in this 2k page
		movlw	low mcall#v(mi)
		call	savertn		; Save return addr
		goto	dest		; and go to subroutine
mcall#v(mi)
		banksel	0
		movfw	b0mwsave	; Set up by mreturn
		endm
;
; User level subroutine return
; 	Converse of the mcall routine
;
mreturn		macro
		goto	restrtn		; Pop stack and return
		endm
;
; Long version of mreturn
;
mlreturn	macro
		lgoto	restrtn		; Pop stack and return
		endm
;
; User level subroutine return
;	Converse of mcall routine
; 	Semantially equivalent to retlw
;
mretlw		macro	rvalue		;
		movlw	rvalue		;
		goto	restrtn		;
		endm
;
; Long version of mretlw
;
mlretlw		macro	rvalue		;
		movlw	rvalue		;
		lgoto	restrtn		;
		endm
;
; Used to set contents of w into the ra control latch
;
ra_out		macro
		movwf	PORTB		; Stuff value into port B
		bsf	PORTC, RENABLE	; Set the RA latch bit	
		bcf	PORTC, RENABLE	; now clear it
		endm			;
;
; Used to set the contents of w into the dec control latch
;
dec_out		macro
		movwf	PORTB		; Stuff value into Port B
		bsf	PORTC, DENABLE	; Set the dec latch bit	
		bcf	PORTC, DENABLE	; now clear it
		endm
;
; RA Step
; 	w = direction, amount.  See FSTEPF, et al.
;	trashes w, temp
;
rstep		macro
		addwf	rtrajp, f	; see where to go next
		movlw	0x1f		; Wrap pointer
		andwf	rtrajp, w	; w gets ptr for gettraj
		movwf	rtrajp		; Save new ptr
		call	gettraj		; Get the table value
		ra_out			; Send to motor
		endm			;			
;
; Dec Step
; 	w = direction, amount.  See FSTEPF, et al.
;	trashes w, temp
;
dstep		macro
		addwf	dtrajp, f	; see where to go next
		movlw	0x1f		; Wrap pointer
		andwf	dtrajp, w	; w gets ptr for gettraj
		movwf	dtrajp		; Save new ptr
		call	gettraj		; Get the table value
		dec_out			; Send to motor
		endm			;
;
; Used to toggle the state of the LED
;
toggle_led	macro
		btfsc	PORTC, LED	;
		goto	$+3		;
		bsf	PORTC, LED	; Set the LED
		goto	$+2
		bcf	PORTC, LED	; Clear it instead		
		endm
;
; Old style unix internals macro names for stopping, allowing interrupts
;    Screwing around with interrupts disabled will ruin your timing
;    Don't use spl0 in interrupt routines, retfie will do it
;
splh		macro
		bcf	INTCON, GIE	;
		endm
;
spl0		macro
		bsf	INTCON, GIE	;
		endm
		list

;
; Global definitions
; Debug
; define SIM
;
#define ClkFreq 4000000			; input clock frequency = 4 MHz
#define baud(X) ((10*ClkFreq/(16*X))+5)/10 - 1
; Low speed tracking constants
#ifdef SIM
#define TRACKH	2	
#else
#define	TRACKH  high 	10000
#endif
#define TRACKL	low 	10000
#define	GUIDEBH	high	13000
#define	GUIDEBL	low	13000
#define	GUIDEFH	high	 7000
#define	GUIDEFL	low	 7000
#define LUNARH	high	 9651
#define	LUNARL	low	 9651
#define	SOLARH	high	10028
#define	SOLARL	low	10028
; High speed indexes (8x, 32x) into table
#define	MEDIDX	4
#define HIGHIDX	64
;
; Global equates
;
CMDLEN		equ	30		; 30 bytes in cmdbuff
MINBATT		equ	144		; Limit for battery ATOD value (10V)
STARTBATT	equ	173		; 12V starting value
;
; Arguments to rstep and dstep.  These depend on the traj table format
USTEPF		equ	1		; microstep forward
USTEPB		equ	-1		; microstep back
HSTEPF		equ	4		; half step forward
HSTEPB		equ	-4		; half step back
FSTEPF		equ	8		; full step forward
FSTEPB		equ	-8		; full step back
;
; Bits for rstatus, dstatus
;
ST_STOP		equ	0		; Stop issued
ST_STOPPED	equ	1		; Stopped
;
; Bits for slew
;
SL_FAST		equ	0		; Fast slew
SL_FAST_MASK	equ	0x01		;
SL_MED		equ	1		; Medium slew
SL_MED_MASK	equ	0x02		;
SL_GUIDE	equ	2		; Guide
SL_GUIDE_MASK	equ	0x04		;
SL_SLOW		equ	3		; Any of Track, lunar, solar
SL_SLOW_MASK	equ	0x08		;
;
; Port A
BATT		equ	0		; RA0 - Analog input to the pot attached to +12
LIM1		equ	2		; RA2 - Digital input: Limit switch on RA, moving West	
LIM2		equ	3		; RA3 - Digital input: Limit switch on RA, moving East
;
; Port C
LED		equ	2		; RC2 - Output: LED
LEDMASK		equ	0x04		; Mask for LED
SHDN		equ	3		; RC3 - Output: SHDN on MAX222
DENABLE		equ	4		; RC4 - Output: Enable Dec. motors latch
RENABLE		equ	5		; RC5 - Output: Enable RA motors latch
;
; RS-232
TXIF		equ	4		; Transmit buffer is empty if set
RCIF		equ	5		; Receive buffer is full if set
;
; Variables use in interrupt context saving
; 
w_temp		EQU	0x20		; variable used for context saving 
status_temp	EQU     w_temp+1	; variable used for context saving
temp_temp	EQU	status_temp+1	; save temp
temp1_temp	EQU	temp_temp+1	; Save temp1
pclath_temp	EQU	temp1_temp+1	; save pclath
int_end		EQU	pclath_temp+1	; Place holder for next interrupt vbl
;
; General globals
;
rapos		equ	int_end		; Long word to hold the ra count
dpos		equ	rapos+4		; Long word to hold the dec count
rtrajp		equ	dpos+4		; Trajectory pointer for ra drive
dtrajp		equ	rtrajp+1	; Trajectory pointer for dec drive
rstatus		equ	dtrajp+1	; Status for RA drive (Between slew cmd & interrupts)
dstatus		equ	rstatus+1	; Status for Dec drive
rinc		equ	dstatus+1	; Increment direction for RA - int level
dinc		equ	rinc+1		; Increment direction for Dec - int level
rspdhigh	equ	dinc+1		; RA speed high byte for int. routines
rspdlow		equ	rspdhigh+1	; RA speed low byte for int. routines
dspdhigh	equ	rspdlow+1	; Dec speed high byte for int. routines
dspdlow		equ	dspdhigh+1	; Dec speed low byte for int. routines
rac_cnt		equ	dspdlow+1	; Accel count passed back from getaccel
dac_cnt		equ	rac_cnt+1	; Accel count passed back from getaccel
stepcmd		equ	dac_cnt+1	; Motor step command -  command level
inccmd		equ	stepcmd+1	; Increment direction - command level
t1stepcmd	equ	inccmd+1	; Motor step command - t1 interrupt level
t2stepcmd	equ	t1stepcmd+1	; Motor step command - t2 interrupt level
slew		equ	t2stepcmd+1	; Communicates between set and slew cmds
temp		equ	slew+1		; Temp register to be used local to any routine
temp1		equ	temp+1		; Another temp
temp2		equ	temp1+1		; And another temp
temp3		equ	temp2+1		; Yet another temp
speedh		equ	temp3+1		; High byte of speed decoded in set speed cmd (or index)
speedl		equ	speedh+1	; Low byte of speed decoded in set speed cmd
rcuridx		equ	speedl+1	; RA Current acceleration index (interrupt level)
dcuridx		equ	rcuridx+1	; Dec Current acceleration index
rhiacidx	equ	dcuridx+1	; RA high acceleration index (interrupt level)
dhiacidx	equ	rhiacidx+1	; Dec high acceleration index (interrupt level)
t1state		equ	dhiacidx+1	; State byte for T1 CCPR1 interrupt
t2state		equ	t1state+1	; State byte for T1 CCPR2 interrupt
d0		equ	t2state+1	; 4 byte registers for fp routines
d1		equ	d0+4		;
d2		equ	d1+4		;
b0mwsave	equ	d2+4		; Bank 0 save for w reg - mcall
avgbatt		equ	b0mwsave+1	; Saves the average battery value
timbatt		equ	avgbatt+1	; Last time the battery was checked
t0ticks		equ	timbatt+1	; Incremented on each tick of t0
cmdidx		equ	t0ticks+1	; Index into cmdbuff
cmdbuff		equ	cmdidx+1	; Buffer for commands
gbl_end		equ	cmdbuff+CMDLEN	; End of globals
;
; Bank 2 stuff - msave, mreturn
;
mlsave		equ	H'00FF'		; Top register in 2nd bank - holds rtn
mfsave		equ	mlsave-1	; Save for FSR
stackptr	equ	mfsave-1	; Save for stack pointer
stack		equ	stackptr-1	; Bottom of stack, grows down
;
; Simulator stuff
	ifdef	SIM
fakeidx		equ	gbl_end		;
	endif
;	
;
; Startup code
;                                                                 
		ORG	0x000		; processor reset vector
		clrf    PCLATH		; ensure page bits are claared
		lgoto    main		; go to beginning of program
;
; Interrupt routines
;
		ORG     0x004		; interrupt vector location
		movwf   w_temp		; save off current W register contents
		movf	STATUS,w	; move status register into W register
		bcf     STATUS,RP0	; ensure file register bank set to 0
		movwf	status_temp	; save off contents of STATUS register
		movfw	temp		; Save temp
		movwf	temp_temp	;
		movfw	temp1		;
		movwf	temp1_temp	;
		movfw	PCLATH		; Save PCLATH
		movwf	pclath_temp	;
; prolog end
		clrf	PCLATH		; Force bank 0
		btfss	PIR1, 2		; Check for timer 1 compare (CCP1IF) match first
		goto	int1		;
; t1 interrupt
t1int
		bcf	PIR1, 2		; Clear CCP1IF
		movlw	high t1jmptbl	;
		movwf	PCLATH		;
		movfw	t1state		; Pick up jump vector
		addwf	PCL, f		; Go
t1jmptbl
		goto	t1stepxof8	; 0 Step 2
		goto	t1stepxof8	; 1 Step 3
		goto	t1stepxof8	; 2 Step 4
		goto	t1stepxof8	; 3 Step 5
		goto	t1stepxof8	; 4 Step 6
		goto	t1stepxof8	; 5 Step 7
		goto	t1step8of8	; 6 Step 8
		goto	t1step1of8	; 7 Step 1
		goto	t1accelxof8	; 8 Step 2
		goto	t1accelxof8	; 9 Step 3
		goto	t1accelxof8	; 10 Step 4
		goto	t1accelxof8	; 11 Step 5
		goto	t1accelxof8	; 12 Step 6
		goto	t1accelxof8	; 13 Step 7
		goto	t1accel8of8	; 14 Step 8
		goto	t1accel1of8	; 15 Step 1
		goto	t1hsxof8	; 16 Step 1
		goto	t1hsxof8	; 17 Step 2
		goto	t1hsxof8	; 18 Step 3
		goto	t1hsxof8	; 19 Step 4
		goto	t1hsxof8	; 20 Step 5
		goto	t1hsxof8	; 21 Step 6
		goto	t1hsxof8	; 22 Step 7
		goto	t1hs8of8	; 23 Step 8
		goto	t1decel1of8	; 24 Step 1
		goto	t1decelxof8	; 25 Step 2
		goto	t1decelxof8	; 26 Step 3
		goto	t1decelxof8	; 27 Step 4
		goto	t1decelxof8	; 28 Step 5
		goto	t1decelxof8	; 29 Step 6
		goto	t1decelxof8	; 30 Step 7
		goto	t1decel8of8	; 31 Step 8
t1stepxof8
		movfw	t1stepcmd	;
		rstep			; Step RA Axis
		incf	t1state, f	;
		goto	t1intend	;
t1step8of8
		movfw	t1stepcmd	; 8th step
		rstep			;
		incf	t1state, f	; Set to step 1
		movfw	rinc		; increment
		addwd	rapos		; Advance rapos - 8 microsteps done
		toggle_led		; Flip LED - full step done
		goto	t1intend	;
t1step1of8
		btfss	rstatus, ST_STOP ; Do they want to stop?
		goto	t1step1of8a	; Skip over if not
t1wrap ; Code to shut off timer 1
		clrf	rstatus		; OK, let's stop...
		bsf	rstatus, ST_STOPPED ;
		movlw	B'00000000'	; CCP1X:CCP1Y=0 CCP1M3:CCP1M0=0
		movwf	CCP1CON		; Shut off compare on match
		banksel	PIE1		; 
		bcf	PIE1, 2		; Turn off CCP1IE
		banksel	0		;
		goto	int1		; Process rest of interrupts
t1step1of8a	movfw	t1stepcmd	;
		rstep			; 1st of 8 microsteps
		movlw	0		; State 0 next
		movwf	t1state		;
		goto	t1intend	;
t1accelxof8	movfw	t1stepcmd	; Step it
		rstep			;
		incf	t1state, f	; Advance nicely along...
		goto	t1intend	;
t1accel8of8
		movfw	t1stepcmd	; 8th step
		rstep			;
		incf	t1state, f	; Set to step 1
		movfw	rinc		; Account for a full step
		addwd	rapos		;
		toggle_led		; We were here
		btfsc	rstatus, ST_STOP ; Stop before we got there?
		goto	t1accel8of8b	;
		decfsz	rac_cnt, f	; More steps at this speed?
		goto	t1intend	; Go on if not
; Accelerating, check next step
		movfw	rcuridx		;
		subwf	rhiacidx, w	; At top speed?
		skpnz			;
		goto	t1accel8of8a	; change state if yes
		incf	rcuridx, f	; Point at next one
		movfw	rcuridx	;
		call	rgetaccel	; Reload CCPRx, rac_cnt
		goto	t1intend	; wrap it up
t1accel8of8a	movlw	16		; State t1hs1of8 next
		movwf	t1state		;
		goto	t1intend	; Wrap it up
t1accel8of8b ; Wants to stop before we hit full speed
		movfw	rcuridx		; Where are we?
		skpnz			;
		goto	t1intend	; Pick it up in t1accel1of8 if done
		decf	rcuridx, f	; Head back down if not
		movfw	rcuridx		;
		call	rgetaccel	; Load "down 1" index
		movlw	24		; decel state - t1decel1of8
		movwf	t1state		;
		goto	t1intend	;
t1accel1of8
		btfss	rstatus, ST_STOP ; Do they want to stop?
		goto	t1accel1of8a	; Skip over if not
		movfw	rcuridx		; Probably 0 if we are here (see t1accel8of8b)
		skpnz			; Wrap it up if yes
		goto	t1wrap		; Will see it in t1accel8of8 then decel if not
t1accel1of8a	movfw	t1stepcmd	;
		rstep			; 1st of 8 microsteps
		movlw	8		; State 8 - t1accelxof8 next
		movwf	t1state		;
		goto	t1intend	;
t1hsxof8
		movfw	t1stepcmd	; Step it
		rstep			;
		incf	t1state, f	; Next state
		goto	t1intend	;
t1hs8of8
		movfw	t1stepcmd	; 8th step
		rstep			;
		movlw	16		; Set to t1hsxof8
		movwf	t1state		;
		movfw	rinc		; Account for a full step
		addwd	rapos		;
		toggle_led		; We were here
;	ifdef	SIM
;		incf	fakeidx, f	; Advance past the 0
;	endif
		btfss	rstatus, ST_STOP ; Stop now?
		goto	t1intend	; Done if not
; Want to stop in high speed run case
; Assume that since we are in the hsrun states, curindex is not 0
		decf	rcuridx, f	; Head back down
		movfw	rcuridx		;
		call	rgetaccel	; Load "down 1" index
		movlw	24		; Next is t1decel1of8
		movwf	t1state		;
		goto	t1intend	;
t1decel1of8
		movfw	rcuridx	; Pick up index
		skpnz			; Wrap it if now 0
		goto	t1wrap		;
		movfw	t1stepcmd	; 1st step of 8
		rstep			;
		incf	t1state, f	; Advance to next state
		goto	t1intend	;
t1decelxof8
		movfw	t1stepcmd	; nth step
		rstep			;
		incf	t1state, f	;
		goto	t1intend	;
t1decel8of8
		movfw	t1stepcmd	; 8th step
		rstep			;
		movlw	24		; Step 1 is next
		movwf	t1state		;
		movfw	rinc		; Account for a full step
		addwd	rapos		;
		toggle_led		; We were here
		decfsz	rac_cnt, f	; More steps at this speed?
		goto	t1intend	; Go on if not
; Decelerating, check next step
		decf	rcuridx, f	;
		movfw	rcuridx		;
		call	rgetaccel	; Reload CCPRx, rac_cnt
		goto	t1intend	; wrap it up
t1intend ; Set up CCPR1 for next interrupt, if not already done
		movfw	rspdlow		; Advance to next time to stop
		addwf	CCPR1L, f	;
		skpnc			;
		incf	CCPR1H, f	;
		movfw	rspdhigh	;
		addwf	CCPR1H, f	;
		bcf	PIR1, 2		; Clear CCP1IF just in case
int1 ; Process rest of interrupts
		btfss	PIR2, 0		; Check for timer 1 compare (CCP2IF) match next
		goto	int2		;
; t2 interrupt
t2int
		bcf	PIR2, 0		; Clear CCP2IF
		movlw	high t2jmptbl	;
		movwf	PCLATH		;
		movfw	t2state		; Pick up jump vector
		addwf	PCL, f		; Go
t2jmptbl
		goto	t2stepxof8	; 0 Step 2
		goto	t2stepxof8	; 1 Step 3
		goto	t2stepxof8	; 2 Step 4
		goto	t2stepxof8	; 3 Step 5
		goto	t2stepxof8	; 4 Step 6
		goto	t2stepxof8	; 5 Step 7
		goto	t2step8of8	; 6 Step 8
		goto	t2step1of8	; 7 Step 1
		goto	t2accelxof8	; 8 Step 2
		goto	t2accelxof8	; 9 Step 3
		goto	t2accelxof8	; 10 Step 4
		goto	t2accelxof8	; 11 Step 5
		goto	t2accelxof8	; 12 Step 6
		goto	t2accelxof8	; 13 Step 7
		goto	t2accel8of8	; 14 Step 8
		goto	t2accel1of8	; 15 Step 1
		goto	t2hsxof8	; 16 Step 1
		goto	t2hsxof8	; 17 Step 2
		goto	t2hsxof8	; 18 Step 3
		goto	t2hsxof8	; 19 Step 4
		goto	t2hsxof8	; 20 Step 5
		goto	t2hsxof8	; 21 Step 6
		goto	t2hsxof8	; 22 Step 7
		goto	t2hs8of8	; 23 Step 8
		goto	t2decel1of8	; 24 Step 1
		goto	t2decelxof8	; 25 Step 2
		goto	t2decelxof8	; 26 Step 3
		goto	t2decelxof8	; 27 Step 4
		goto	t2decelxof8	; 28 Step 5
		goto	t2decelxof8	; 29 Step 6
		goto	t2decelxof8	; 30 Step 7
		goto	t2decel8of8	; 31 Step 8
t2stepxof8
		movfw	t2stepcmd	;
		dstep			; Step RA Axis
		incf	t2state, f	;
		goto	t2intend	;
t2step8of8
		movfw	t2stepcmd	; 8th step
		dstep			;
		incf	t2state, f	; Set to step 1
		movfw	dinc		; increment
		addwd	dpos		; Advance dpos - 8 microsteps done
		toggle_led		; Flip LED - full step done
		goto	t2intend	;
t2step1of8
		btfss	dstatus, ST_STOP ; Do they want to stop?
		goto	t2step1of8a	; Skip over if not
t2wrap ; Code to shut off timer 1
		clrf	dstatus		; OK, let's stop...
		bsf	dstatus, ST_STOPPED ;
		banksel	PIE2		; 
		bcf	PIE2, 0		; Turn off CCP2IE
		banksel	0		;
		movlw	B'00000000'	; CCP1X:CCP1Y=0 CCP1M3:CCP1M0=0
		movwf	CCP2CON		; Shut off compare on match
		goto	int2		; Process rest of interrupts
t2step1of8a	movfw	t2stepcmd	;
		dstep			; 1st of 8 microsteps
		movlw	0		; State 0 next
		movwf	t2state		;
		goto	t2intend	;
t2accelxof8
		movfw	t2stepcmd	; Step it
		dstep			;
		incf	t2state, f	; Advance nicely along...
		goto	t2intend	;
t2accel8of8
		movfw	t2stepcmd	; 8th step
		dstep			;
		incf	t2state, f	; Set to step 1
		movfw	dinc		; Account for a full step
		addwd	dpos		;
		toggle_led		; We were here
		btfsc	dstatus, ST_STOP ; Stop before we got there?
		goto	t2accel8of8b	;
		decfsz	dac_cnt, f	; More steps at this speed?
		goto	t2intend	; Go on if not
; Accelerating, check next step
		movfw	dcuridx		;
		subwf	dhiacidx, w	; At top speed?
		skpnz			;
		goto	t2accel8of8a	; change state if yes
		incf	dcuridx, f	; Point at next one
		movfw	dcuridx		;
		call	dgetaccel	; Reload CCPRx, dac_cnt
		goto	t2intend	; wrap it up
t2accel8of8a	movlw	16		; State t2hs1of8 next
		movwf	t2state		;
		goto	t2intend	; Wrap it up
t2accel8of8b ; Wants to stop before we hit full speed
		movfw	dcuridx		; Where are we?
		skpnz			;
		goto	t2intend	; Pick it up in t2accel1of8 if done
		decf	dcuridx, f	; Head back down if not
		movfw	dcuridx	;
		call	dgetaccel	; Load "down 1" index
		movlw	24		; decel state - t2decel1of8
		movwf	t2state		;
		goto	t2intend	;
t2accel1of8
		btfss	dstatus, ST_STOP ; Do they want to stop?
		goto	t2accel1of8a	; Skip over if not
		movfw	dcuridx		; Probably 0 if we are here (see t2accel8of8b)
		skpnz			; Wrap it up if yes
		goto	t2wrap		; Will see it in t2accel8of8 then decel if not
t2accel1of8a	movfw	t2stepcmd	;
		dstep			; 1st of 8 microsteps
		movlw	8		; State 8 - t2accelxof8 next
		movwf	t2state		;
		goto	t2intend	;
t2hsxof8
		movfw	t2stepcmd	; Step it
		dstep			;
		incf	t2state, f	; Next state
		goto	t2intend	;
t2hs8of8
		movfw	t2stepcmd	; 8th step
		dstep			;
		movlw	16		; Set to t2hsxof8
		movwf	t2state		;
		movfw	dinc		; Account for a full step
		addwd	dpos		;
		toggle_led		; We were here
	ifdef	SIM
		incf	fakeidx, f	; Advance past the 0
	endif
		btfss	dstatus, ST_STOP ; Stop now?
		goto	t2intend	; Done if not
; Want to stop in high speed run case
; Assume that since we are in the hsrun states, curindex is not 0
		decf	dcuridx, f	; Head back down
		movfw	dcuridx		;
		call	dgetaccel	; Load "down 1" index
		movlw	24		; Next is t2decel1of8
		movwf	t2state		;
		goto	t2intend	;
t2decel1of8
		movfw	dcuridx		; Pick up index
		skpnz			; Wrap it if now 0
		goto	t2wrap		;
		movfw	t2stepcmd	; 1st step of 8
		dstep			;
		incf	t2state, f	; Advance to next state
		goto	t2intend	;
t2decelxof8
		movfw	t2stepcmd	; nth step
		dstep			;
		incf	t2state, f	;
		goto	t2intend	;
t2decel8of8
		movfw	t2stepcmd	; 8th step
		dstep			;
		movlw	24		; Step 1 is next
		movwf	t2state		;
		movfw	dinc		; Account for a full step
		addwd	dpos		;
		toggle_led		; We were here
		decfsz	dac_cnt, f	; More steps at this speed?
		goto	t2intend	; Go on if not
; Decelerating, check next step
		decf	dcuridx, f	;
		movfw	dcuridx		;
		call	dgetaccel	; Reload CCPRx, dac_cnt
		goto	t2intend	; wrap it up
t2intend ; Set up CCPR2 for next interrupt, if not already done
		movfw	dspdlow		; Advance to next time to stop
		addwf	CCPR2L, f	;
		skpnc			;
		incf	CCPR2H, f	;
		movfw	dspdhigh	;
		addwf	CCPR2H, f	;
		bcf	PIR2, 0		; Clear CCP2IF just in case
int2 ; Check for timer 0
		btfss	INTCON, 2	; Did timer 0 fire?
		goto	int3		; Skip if not
		bcf	INTCON, 2	; Clear flag if it did
		incf	t0ticks, f	; Advance timer
int3 ;
int4 ; Wrap up interrupt processing
; post processing - match up with prolog
		bcf     STATUS,RP0	; ensure file register bank set to 0
		movfw	pclath_temp	; restore PCLATH
		movwf	PCLATH		;
		movfw	temp1_temp	; restore temp1
		movwf	temp1		;
		movfw	temp_temp	; restore temp
		movwf	temp		;
		movf    status_temp,w	; retrieve copy of STATUS register
		movwf	STATUS		; restore pre-isr STATUS register contents
		swapf   w_temp,f	;
		swapf   w_temp,w	; restore pre-isr W register contents
		retfie			; return from interrupt
;
; gettraj
;	w = index into table
;	returns the value from the table
; 	Org up to a reasonable boundary
		org	($+0x100) & 0x3F00
gettraj
		movwf	temp		; Stash the index
		movlw	high traj	;
		movwf	PCLATH		;
		movfw	temp		;
		addwf	PCL, f		; assumes same page, etc
;
; Trajectory table for microstepping
; Numbers in the comments column give the number of microstep, half and
; full step.  The top nibble goes to the 1st controller, the bottom to
; the second.  Of the 4 bits in a nibble, the MSB is the phase, the bottom
; 3 are the digital code.
;
traj		dt	H'CC'		;  0 0 0
		dt	H'BD'		;  1
		dt	H'AE'		;  2 
		dt	H'9F'		;  3
		dt	H'0F'		;  4 1
		dt	H'1F'		;  5
		dt	H'2E'		;  6
		dt	H'3D'		;  7
		dt	H'4C'		;  8 2 1
		dt	H'5B'		;  9 
		dt	H'6A'		; 10
		dt	H'79'		; 11
		dt	H'70'		; 12 3
		dt	H'71'		; 13
		dt	H'62'		; 14
		dt	H'53'		; 15
		dt	H'44'		; 16 4 2
		dt	H'35'		; 17
		dt	H'26'		; 18
		dt	H'17'		; 19
		dt	H'87'		; 20 5
		dt	H'97'		; 21
		dt	H'A6'		; 22
		dt	H'B5'		; 23
		dt	H'C4'		; 24 6 3
		dt	H'D3'		; 25
		dt	H'E2'		; 26
		dt	H'F1'		; 27
		dt	H'F8'		; 28 7
		dt	H'F9'		; 29
		dt	H'EA'		; 30
		dt	H'DB'		; 31

;
; GetAccel
;	Gets the accel. at index w
;	Loads rac_cnt, rspdlow, rspdhigh
;	Got a version for ra and one for dec
;
rgetaccel
		clrf	temp1		; Use temp, temp1 for index
		movwf	temp		; Multiply w by 3 to index into tbl
		clrc			; Clear carry
		rlf	temp, f		; Shift index left
		rlf	temp1, f	;
		addwf	temp, f		; w*3 now in temp, temp1
		addcf	temp1, f	;
		call	getacc1		; Get count
		movwf	rac_cnt		;
		movlw	1		;
		addwf	temp, f		;
		addcf	temp1, f	;
		call	getacc1		; Get msb
		movwf	rspdhigh	;
		movlw	1
		addwf	temp, f		;
		addcf	temp1, f	;
		call	getacc1		; Get lsb
		movwf	rspdlow		;
		return
dgetaccel
		clrf	temp1		; Use temp, temp1 for index
		movwf	temp		; Multiply w by 3 to index into tbl
		clrc			; Clear carry
		rlf	temp, f		; Shift index left
		rlf	temp1, f	;
		addwf	temp, f		; w*3 now in temp, temp1
		addcf	temp1, f	;
		call	getacc1		; Get count
		movwf	dac_cnt		;
		movlw	1		;
		addwf	temp, f		;
		addcf	temp1, f	;
		call	getacc1		; Get msb
		movwf	dspdhigh	;
		movlw	1
		addwf	temp, f		;
		addcf	temp1, f	;
		call	getacc1		; Get lsb
		movwf	dspdlow		;
		return
getacc1
		movlw	high accel	; set PCLATH for accel
		movwf	PCLATH		;
		movfw	temp1		; Add in top of index
		addwf	PCLATH, f	;
		movfw	temp		; Now do bottom
		addlw	low accel	;
		addcf	PCLATH, f	; prop. carry
		movwf	PCL		; and go
		
;
; Acceleration table
; 	Each entry has a number of 16 bit microsecond counts
;	Accel. at the rate of 1250 full steps/second/second
;	These are microstep counts.
;
accel
		dt	 1 ,  H'13',   H'88'
		dt	 1 ,  H'08',   H'17'
		dt	 1 ,  H'06',   H'35'
		dt	 1 ,  H'05',   H'3C'
		dt	 1 ,  H'04',   H'9D' ; 100 fs/sec = 8x
		dt	 1 ,  H'04',   H'2B'
		dt	 1 ,  H'03',   H'D6'
		dt	 1 ,  H'03',   H'92'
		dt	 1 ,  H'03',   H'5A'
		dt	 1 ,  H'03',   H'2C'
		dt	 1 ,  H'03',   H'04'
		dt	 1 ,  H'02',   H'E2'
		dt	 1 ,  H'02',   H'C3'
		dt	 1 ,  H'02',   H'A9'
		dt	 1 ,  H'02',   H'91'
		dt	 1 ,  H'02',   H'7B'
		dt	 1 ,  H'02',   H'68'
		dt	 1 ,  H'02',   H'56'
		dt	 1 ,  H'02',   H'46'
		dt	 1 ,  H'02',   H'36'
		dt	 1 ,  H'02',   H'28'
		dt	 1 ,  H'02',   H'1B'
		dt	 1 ,  H'02',   H'0F'
		dt	 1 ,  H'02',   H'04'
		dt	 1 ,  H'01',   H'F9'
		dt	 1 ,  H'01',   H'EF'
		dt	 1 ,  H'01',   H'E6'
		dt	 1 ,  H'01',   H'DD'
		dt	 1 ,  H'01',   H'D5'
		dt	 1 ,  H'01',   H'CD'
		dt	 1 ,  H'01',   H'C5'
		dt	 1 ,  H'01',   H'BE'
		dt	 1 ,  H'01',   H'B7'
		dt	 1 ,  H'01',   H'B0'
		dt	 1 ,  H'01',   H'AA'
		dt	 1 ,  H'01',   H'A4'
		dt	 1 ,  H'01',   H'9E'
		dt	 1 ,  H'01',   H'99'
		dt	 1 ,  H'01',   H'93'
		dt	 1 ,  H'01',   H'8E'
		dt	 1 ,  H'01',   H'89'
		dt	 1 ,  H'01',   H'84'
		dt	 1 ,  H'01',   H'80'
		dt	 1 ,  H'01',   H'7B'
		dt	 1 ,  H'01',   H'77'
		dt	 1 ,  H'01',   H'73'
		dt	 1 ,  H'01',   H'6F'
		dt	 1 ,  H'01',   H'6B'
		dt	 1 ,  H'01',   H'67'
		dt	 1 ,  H'01',   H'64'
		dt	 1 ,  H'01',   H'60'
		dt	 1 ,  H'01',   H'5D'
		dt	 1 ,  H'01',   H'59'
		dt	 1 ,  H'01',   H'56'
		dt	 1 ,  H'01',   H'53'
		dt	 1 ,  H'01',   H'50'
		dt	 1 ,  H'01',   H'4D'
		dt	 1 ,  H'01',   H'4A'
		dt	 1 ,  H'01',   H'47'
		dt	 1 ,  H'01',   H'44'
		dt	 1 ,  H'01',   H'42'
		dt	 1 ,  H'01',   H'3F'
		dt	 1 ,  H'01',   H'3C'
		dt	 1 ,  H'01',   H'3A'
		dt	 1 ,  H'01',   H'38' ; 400 fs/sec = 32x

		org	0x800

	ifdef SIM
fakeinput
		movfw	fakeidx		; Pick up the index
		call	getfake		;
		andlw	0xff		; Zero?
		skpz			;
		incf	fakeidx, f	; Always return 0 if it is
		mreturn			; 
getfake
		movwf	temp
		movlw	high faketbl
		movwf	PCLATH
		movfw	temp
		addlw	low faketbl	; Index into table
		movwf	PCL		; assumes same page, etc
faketbl		dt	"GB\r"
		dt	0
		dt	"Q\r"
		dt	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0	
	endif
;
; Speed tbl
;	Call with index in temp
;	Returns char at that index in w
;
getspeed
		movlw	high speedtbl	;
		movwf	PCLATH		;
		movfw	temp		;
		addlw	low speedtbl	; Index into table
		movwf	PCL		; assumes same page, etc
speedtbl
		dt	'T', SL_SLOW_MASK, TRACKH, TRACKL
		dt	'G', SL_GUIDE_MASK, 0, 0
		dt	'L', SL_SLOW_MASK, LUNARH, LUNARL
		dt	'S', SL_SLOW_MASK, SOLARH, SOLARL
		dt	'M', SL_MED_MASK, MEDIDX, 0
		dt	'F', SL_FAST_MASK, HIGHIDX, 0
		dt	0
;
; Powers of 10 table
;	Used in dtoa to convert digital to ascii
;	Gets byte at temp, increments temp
;
getpot
		movlw	high pottbl	;
		movwf	PCLATH		;
		movfw	temp		;
		incf	temp, f		; Point to next one
		addlw	low pottbl	;
		movwf	PCL		;
pottbl
		dt	59, 154, 202,   0	; 1000000000
		dt	 5, 245, 225,   0	; 100000000
		dt	 0, 152, 150, 128	; 10000000
		dt	 0,  15,  66,  64	; 1000000
		dt	 0,   1, 134, 160	; 100000
		dt	 0,   0,  39,  16	; 10000
		dt	 0,   0,   3, 232	; 1000
		dt	 0,   0,   0, 100	; 100
		dt	 0,   0,   0,  10	; 10
		dt	 0,   0,   0,   1	; 1
;
; User level subroutine call 
; 	call with:
;	  PCLATH set to return addr top
;	  w set to return addr bottom
;	  b0mwsave set to calling w
;
savertn
		banksel	stackptr	;
		movwf	mlsave		; Save bottom of return addr
		movfw	FSR		; Save FSR
		movwf	mfsave		;
		movfw	stackptr	; Pick up stack pointer
		movwf	FSR		;
		movfw	mlsave		; Pick up bottom
		movwf	INDF		; Push on stack
		decf	FSR, f		;
		movfw	PCLATH		; Pick up top
		movwf	INDF		; Push on stack
		decf	FSR, f		;
		movfw	FSR		; Stash stack pointer
		movwf	stackptr	;
		movfw	mfsave		; Restore FSR
		movwf	FSR		;
		banksel	0		; Back to bank 0
		movfw	b0mwsave	;
		return			; All done
;
; User level subroutine return
;	Just jump here
;
restrtn
		movwf	b0mwsave	; Save w
		banksel	stackptr	;
		movfw	FSR		; Save FSR
		movwf	mfsave		;
		movfw	stackptr	; Load FSR from stack pointer
		movwf	FSR		;
		incf	FSR, f		; Back to saved hi addr
		movfw	INDF		;
		movwf	PCLATH		; Save in PCLATH
		incf	FSR, f		; Back to saved low addr
		movfw	INDF		;
		movwf	mlsave		; Stash for a minute
		movfw	FSR		;
		movwf	stackptr	; Save new stack pointer
		movfw	mfsave		; 
		movwf	FSR		; Restore FSR
		movfw	mlsave		; Pick low addr back up
		movwf	PCL		; And return

;
; Start of code
;
main
; Initalize main registers - shut everything off for now
		movlw	B'10000111'	; RBPU=1 INTEDG=0 TOCS=0 TOSE=0 PSA=0 PS0:PS2=7
		banksel	OPTION_REG	;
		movwf	OPTION_REG	;
		movlw	B'00000000'	; Turn off all interrupts, clear bits
		banksel	INTCON		;
		movwf	INTCON		;
		movlw	B'00000000'	; Turn off all interrupt enables in PIE1
		banksel	PIE1		;
		movwf	PIE1		;
		movlw	B'00000000'	; Clear all interrupt flag bits in PIR1
		banksel	PIR1		;
		movwf	PIR1		;
		movlw	B'00000000'	; Clear CCP2IE in PIE2
		banksel	PIE2		;
		movwf	PIE2		;
		movlw	B'00000000'	; Clear all flags in PIR2
		banksel	PIR2		;
		movwf	PIR2		;
		banksel	0
; Set up user level stack
		banksel	stackptr	;
		movlw	stack		;
		movwf	stackptr	;
		banksel	0
; Initialize port registers
		banksel	PORTA		; Initalize PORTs
		clrf	PORTA		;
		clrf	PORTB		;
		clrf	PORTC		;
		banksel	TRISA		;
		movlw	B'00111111'	; Set all as inputs
		movwf	TRISA		;
		clrf	TRISB		; Set all as outputs
		movlw	B'11000000'	; Transmit, receive = in, others = out
		movwf	TRISC		; 
		banksel	0
; First thing, shut down the motors.
		movlw	0		;
		ra_out			; 0 to RA motor latch
		movlw	0		;
		dec_out			;
; Set up globals that need it
		clrd	rapos		;
		clrd	dpos		;
		clrf	rtrajp		;
		clrf	dtrajp		;
		clrf	cmdidx		; point at first char
		clrf	rstatus		;
		clrf	dstatus		;
		bsf	rstatus, ST_STOPPED ;
		bsf	dstatus, ST_STOPPED ;
		movlw	TRACKH		; default for slew command
		movwf	speedh		;
		movlw	TRACKL		;
		movwf	speedl		;
		movlw	SL_SLOW_MASK	;
		movwf	slew		;
		movlw	USTEPF		;
		movwf	stepcmd		;
		movlw	1		;
		movwf	inccmd		;
		movlw	STARTBATT	; Set up start for rolling average
		movwf	avgbatt		;
		clrf	t0ticks		; No ticks received yet
		clrf	timbatt		; Last time the battery was checked = 0
	ifdef	SIM
		clrf	fakeidx
	endif
; Set up timers
		movlw	B'00000000'	; TICKPS1:TICKPS0=0 T1OSCEN=0 T1SYNC=0 TMR1CS=0
		movwf	T1CON		; Set up for internal clock, leave off for now
		clrf	TMR1L		; Zero out timer 1
		clrf	TMR1H		;
		bsf	T1CON, 0	; Start Timer 1 ticking
		clrf	TMR0		; Clear timer 0
		bcf	INTCON, 2	; Clear T0IF
		bsf	INTCON, 5	; Set T0IE - want an interrupt on each rollover
; Set up a/d
		banksel	ADCON1		;
		movlw	0		; RA0:RA5 all analog inputs
		movwf	ADCON1		;
		banksel	ADCON0		;
		movlw	B'11000000'	; ADCS1:ADCS0=Internal Osc, ADON=converter off
		movwf	ADCON0		;
; Set up RS232
		movlw	baud(9600)	; Should be 25 for 9615 baud
		banksel	SPBRG		;
		movwf	SPBRG		;
		banksel	0		;
		movlw	B'00100100'	; CSRC=0 TX9=0 TXEN=1 SYNC=0 BRGH=1
		banksel	TXSTA		;
		movwf	TXSTA		;
		banksel	0		;
		movlw	B'10010000'	; SPEN=1 RX9=0 SREN=0 CREN=1
		movwf	RCSTA		;
		bsf	PORTC, SHDN	; Enable the MAX222 chip
     		bsf	PORTC, LED	; Set the LED, ready to roll
; Send out a "Hello" message
		mcall	sayhi		;
; Finally, enable interrupts - everything is ready to go
		bsf	INTCON, 7	; Set GIE
		bsf	INTCON, 6	; Set PEIE
; Default mode is track so we don't need a paddle to start
		mcall	dotrack		; Start it tracking
;	
; Main loop
;
loop
		mcall	rs232in		; Try to get a character
		andlw	0xff		; Check to see if we  got one
		skpnz			; 
		goto	main1		;
		mcall	bufchar		; Buffer char if yes
		sublw	0x0D		; Is it a CR?
		skpz
		goto	main1		;
		mcall	docmd		; Do the command if yes
main1
		mcall	chklim		; Check the limit switches
		mcall	chkbatt		; Check the battery voltage
		goto	loop
;
; Hello, we're here msg
;
sayhi
		movlw	high sayhisub	;
		movwf	PCLATH		;
		movlw	0		; Set up index
		movwf	temp		;
sayhi1		movfw	temp		; Pick up the index
		call	sayhisub	; Get the char
		andlw	0xff		; Zero?
		skpnz			;
		goto	sayhidone	; End if done
		mcall	rs232out	; Send the char
		incf	temp, f		; Advance the index
		goto	sayhi1		; Loop
sayhidone	mreturn			;

sayhisub	addwf	PCL, f		; Get a char
		dt	"\r\ndrivera V1.0 7/5/00\r\n>"
		dt	0
;
; Send a character out the RS232 port
;	Character passed in w
;	(Same) character returned in w
rs232out
	IFNDEF	SIM
		btfss	PIR1, TXIF	; Check for buffer full
		goto	rs232out	;
	ENDIF
		movwf	TXREG		; Send the character
		toggle_led		; Let them know we sent one
		mreturn			;
;
; Check for a character in the RS232 input register
;	Returns character in w or 0 if no char
;	Echoes character to output line if we got one
;
rs232in
		movlw	0		; assume no char
	IFDEF	SIM
		mcall	fakeinput	;
	ELSE
		btfss	PIR1, RCIF	; Check receive buffer full
		goto	rs232indone	; Return if no char
		movfw	RCREG		; Get the char
	ENDIF		
		mcall	rs232out	; Echo it
rs232indone	mreturn			; Return the character
;
; Buffer up an ascii character
;	Character passed in w
;	Returns the character in w
;
bufchar
		movwf	temp		; Stash the char
		movfw	cmdidx		; Pick up the index to the next char
		sublw	CMDLEN		; Check length
		skpz			;
		goto	bufchar1	;
		movfw	temp		; don't store if buff is full
		goto	bufchardone	;
bufchar1
		movfw	cmdidx		; Grab it again
		addlw	cmdbuff		; index into buffer
		movwf	FSR		;
		movfw	temp		; Get the character
		movwf	INDF		; indirect store
		incf	cmdidx, f	; Increment the index	
bufchardone	mreturn			; And done
;
; Send buffer at FSR to the output rs232
;	Buffer is null terminated
;
buffout
		movfw	INDF		; Get a char
		skpnz			;
		goto	buffoutend	; Done if null
		mcall	rs232out	; Send the char
		incf	FSR, f		; Advance to the next one
		goto	buffout		; And loop
buffoutend	mreturn
		
;
; Do the command in the buffer
;	No parameters passed
;	No return value
docmd
		movlw	cmdbuff		; Point at the buffer
		movwf	FSR		;
		movfw	INDF		; Pick off the 1st char
		sublw	'F'		; Full step?
		skpz			;
		goto	docmd1		;
		mcall	dofstep		; Do full step cmd
		goto	docmdend	;
docmd1		movfw	INDF		;
		sublw	'G'		; Get?
		skpz			;
		goto	docmd2		;
		mcall	doget		; Do the get commands
		goto	docmdend	;
docmd2		movfw	INDF		;
		sublw	'U'		; Microstep?
		skpz			;
		goto	docmd3		;
		mcall	domicrostep	; Do microstep commands
		goto	docmdend	;	
docmd3		movfw	INDF		;
		sublw	'Q'		; Quit?
		skpz			;
		goto	docmd4		;
		movlw	0		; Quit, don't idle
		mcall	doquit		; Do quit cmd
		goto	docmdend	;
docmd4		movfw	INDF		;
		sublw	'R'		; Reset?
		skpz			;
		goto	docmd5		;
		lgoto 	0x00		; xxx may not be adequate
docmd5		movfw	INDF		;
		sublw	'S'		; Set?
		skpz			;
		goto	docmd6		;
		mcall	doset		; Do Set commands
		goto	docmdend	;
docmd6		movfw	INDF		;
		sublw	'T'		; Track?
		skpz			;
		goto	docmd7		;
		mcall	dotrack		; Do Track cmd
		goto	docmdend	;
docmd7		movfw	INDF		;
		sublw	'M'		; Slew?
		skpz			;
		goto	docmd8		;
		mcall	doslew		; Do Slew cmd
		goto	docmdend	;
docmd8		movfw	INDF		;
		sublw	'I'		; Idle?
		skpz			;
		goto	docmd9		;
		movlw	1		; Quit and idle
		mcall	doquit		; Do Idle command
		goto	docmdend	;
docmd9	; Didn't recognize the cmd...
		movlw	1		; - error invalid command
; Wrap it up, prompt for next cmd
docmdend
		movwf	temp
		movlw	0x0A		; LF
		mcall	rs232out	;
		movfw	temp		; Get command return status
		skpnz			;
		goto	dcmde1		;
		movlw	'?'		; Error indicator
		mcall 	rs232out	;
dcmde1		movlw	'>'		; Prompt
		mcall	rs232out	;
		clrf	cmdidx		; Set the index back to start
		mreturn			;
;
; Start tracking
;	Hard code the speeds and slew...

;
dotrack
		movlw	TRACKH		; Set up parameters for raslowslew
		movwf	speedh		;
		movlw	TRACKL		;
		movwf	speedl		;
		movlw	USTEPB		;
		movwf	stepcmd		;
		movlw	-1		;
		movwf	inccmd
		goto	raslowslew	;
;
; Start slewing
;	Call with FSR pointing to M
;	M<m><dir>
;
doslew
		incf	FSR, f		; Advance to "R" or "D"
		movfw	INDF		;
		sublw	'R'		; RA?
		skpz			;
		goto	doslewnr	;
		movlw	0		; 0 clear = ra
		goto	doslewfb	;
doslewnr	movfw	INDF		;
		sublw	'D'		; Dec?
		skpz			;
		goto	doslewbad	;
		movlw	1		; 0 set = dec
doslewfb	movwf	temp		; bit 0 of temp = ra/dec
		incf	FSR, f		; Advance to '+' or '-'
		movfw	INDF		; Get direction
		sublw	'+'		; Forward?
		skpz			;
		goto	doslewcm	;
		bsf	temp, 1		; 1 set = +
		goto	doslew1
doslewcm	movfw	INDF		;
		sublw	'-'		;
		skpz			;
		goto	doslewbad	; 1 clear = -
doslew1
		btfss	slew, SL_GUIDE	; Guide?
		goto	doslew2		;
		btfss	temp, 1		; Forward, back
		goto	doslew3
		movlw	GUIDEFH		; Set speed to guide forward
		movwf	speedh		;
		movlw	GUIDEFL		;
		movwf	speedl		; 2 clear = slow
		goto	doslew4		;
doslew3		movlw	GUIDEBH		;
		movwf	speedh		;
		movlw	GUIDEBL		;
		movwf	speedl		;
		bsf	temp, 1		; Really forward
doslew2		btfsc	slew, SL_FAST	;
		bsf	temp, 2		; 2 set = fast, 2 clear = slow
		btfsc	slew, SL_MED	;
		bsf	temp, 2		;
doslew4	; Temp all set up
		btfss	temp, 1		; Forward?
		goto	doslew6		;
		movlw	USTEPF		; Yes, forward,  Set up for motor start
		movwf	stepcmd		;
		movlw	1		;
		movwf	inccmd		;
		goto	doslew7		;
doslew6		movlw	USTEPB		; Nope, backwards
		movwf	stepcmd		;
		movlw	-1		;
		movwf	inccmd		;
doslew7		btfsc	temp, 0		; RA set?
		goto	doslew5		;
		btfss	temp, 2		; Fast?
		goto	raslowslew	;
		goto	rafastslew	;
doslew5		btfss	temp, 2		;
		goto	dslowslew	;
		goto	dfastslew	;
doslewbad	mretlw	1		; Error

;
; raslowslew starts a slow slew
;    speedh, speedl, stepcmd, inccmd all set up previously
;
raslowslew
		bsf	rstatus, ST_STOP ; Signal that we need to stop
raslowslew2
		btfss	rstatus, ST_STOPPED ; Check for stopped
		goto	raslowslew2	; Wait for a stop
raslowslew1
		movfw	speedh		; Set up compare regs for int level
		movwf	rspdhigh	;
		movfw	speedl		;
		movwf	rspdlow		;
		movlw	0		; Start in state 0
raslewall ; start up the engine...  Enter here with initial state in w
		movwf	t1state		;
		clrf	rstatus		;
		movfw	inccmd		; Set up interrupt level vbls
		movwf	rinc		;
		movfw	stepcmd		;
		movwf	t1stepcmd	;	
		splh			; Disable ints while we get timer 1
		movfw	TMR1H		; Mark1 - Pick up high
		addwf	rspdhigh, w	; calc capture reg
		movwf	CCPR1H		;
		movlw	5		; Mark2-Mark1+1 timing
		subwf	TMR1L, w	; Mark2 - Pick up low
		spl0			; Reenable
		addwf	rspdlow, w	; Calc other reg
		movwf	CCPR1L		;
		skpnc			;
		incf	CCPR1H, f	;
		movlw	B'00001010'	; CCP1X:CCP1Y=0 CCP1M3:CCP1M0=1010
		movwf	CCP1CON		; Set up for compare mode, int on match
		bcf	PIR1, 2		; Be sure CCP1IF is clear
		banksel	PIE1		;
		bsf	PIE1, 2		; Enable the capture interrupt
		banksel	0		;
; rstep expanded here by hand to allow for lcall of gettraj
		movfw	t1stepcmd	; Step 1 of 8
		addwf	rtrajp, f	; see where to go next
		movlw	0x1f		; Wrap pointer
		andwf	rtrajp, w	; w gets ptr for gettraj
		movwf	rtrajp		; Save new ptr
		lcall	gettraj		; Get the table value
		ra_out			; Send to motor
		mlretlw	0		; Successful completion
;
; rafastslew starts a fast slew
;	speedh, speedl, stepcmd, inccmd all set up previously
;
rafastslew
		bsf	rstatus, ST_STOP ; Signal that we need to stop
rafastslew2
		btfss	rstatus, ST_STOPPED ; Check for stopped
		goto	rafastslew	; Wait for a stop
rafastslew1
		movlw	0		; Start at 0
		movwf	rcuridx	; Set starting point
		lcall	rgetaccel	; Set up rac_cnt, rspdhigh, rspdlow
		movfw	speedh		; Pick up high index
		movwf	rhiacidx	; Set stopping point
		movlw	8		; First state = t1accel
		lgoto	raslewall	;
;
; dslowslew starts a slow slew on dec drive
;    speedh, speedl, stepcmd, inccmd all set up previously
;
dslowslew
		bsf	dstatus, ST_STOP ; Signal that we need to stop
dslowslew2
		btfss	dstatus, ST_STOPPED ; Check for stopped
		goto	dslowslew2	; Wait for a stop
dslowslew1
		movfw	speedh		; Set up compare regs for int level
		movwf	dspdhigh	;
		movfw	speedl		;
		movwf	dspdlow		;
		movlw	0		; Start in state 0
dslewall ; start up the engine...  Enter here with initial state in w
		movwf	t2state		;
		clrf	dstatus		;
		movfw	inccmd		; Set up interrupt level vbls
		movwf	dinc		;
		movfw	stepcmd		;
		movwf	t2stepcmd	;	
		splh			; Disable ints while we get timer 1
		movfw	TMR1H		; Mark1 - Pick up high
		addwf	dspdhigh, w	; calc capture reg
		movwf	CCPR2H		;
		movlw	5		; Mark2-Mark1+1 timing
		subwf	TMR1L, w	; Mark2 - Pick up low
		spl0			; Reenable
		addwf	dspdlow, w	; Calc other reg
		movwf	CCPR2L		;
		skpnc			;
		incf	CCPR2H, f	;
		movlw	B'00001010'	; CCP1X:CCP1Y=0 CCP1M3:CCP1M0=1010
		movwf	CCP2CON		; Set up for compare mode, int on match
		bcf	PIR2, 0		; Be sure CCP2IF is clear
		banksel	PIE2		;
		bsf	PIE2, 0		; Enable the capture interrupt
		banksel	0		;
; dstep expanded here by hand to allow the lcall of gettraj
		movfw	t2stepcmd	; Step 1 of 8
		addwf	dtrajp, f	; see where to go next
		movlw	0x1f		; Wrap pointer
		andwf	dtrajp, w	; w gets ptr for gettraj
		movwf	dtrajp		; Save new ptr
		lcall	gettraj		; Get the table value
		dec_out			; Send to motor
		mlretlw	0		; Successful completion
;
; dfastslew starts a fast slew
;	speedh, speedl, stepcmd, inccmd all set up previously
;
dfastslew
		bsf	dstatus, ST_STOP ; Signal that we need to stop
dfastslew2
		btfss	dstatus, ST_STOPPED ; Check for stopped
		goto	dfastslew	; Wait for a stop
dfastslew1
		movlw	0		; Start at 0
		movwf	dcuridx		; Set starting point
		lcall	dgetaccel	; Set up rac_cnt, rspdhigh, rspdlow
		movfw	speedh		; Pick up high index
		movwf	dhiacidx	; Set stopping point
		movlw	8		; First state = t2accel
		lgoto	dslewall	;		

;
; Full step command
;
dofstep
		mreturn
;
; Micro step command
;
domicrostep
		mreturn
;
; Quit commands
;	w = 0 means just quit, leave motors running
;	w = 1 means quit, shut motors off (idle)
;
doquit
		movwf	temp		; Save parameter
		incf	FSR, f		; Advance to next in cmd
		movfw	INDF		; Get the cmd
		sublw	0x0D		; CR?
		skpz			;
		goto	doquit1		;
; No parameter means both
		movfw	temp		;
		mcall	stopboth	;
		goto	doquitok	;
doquit1
		movfw	INDF		; Maybe it's an R?
		sublw	'R'		;
		skpz			;
		goto	doquit3		;
		bsf	rstatus, ST_STOP ; Signal RA to stop
doquit2		btfss	rstatus, ST_STOPPED ; Wait for RA to stop
		goto	doquit2		;
		movfw	temp		; Idle too?
		skpnz			;
		goto	doquitok	; Done, RA stopped
		movlw	0		;
		ra_out			; Kill power to the motor
		goto	doquitok	;
doquit3
		movfw	INDF		; OK, got to be a D...
		sublw	'D'		;
		skpnz			; Return if not
		goto	doquit5		;
		mretlw	1		; Error return
doquit5		bsf	dstatus, ST_STOP ; Signal DEC to stop
doquit4		btfss	dstatus, ST_STOPPED ;
		goto	doquit4		;
		movfw	temp		; Idle too?
		skpnz			;
		goto	doquitok	; Done, DEC stopped
		movlw	0		;
		dec_out			; Kill power to the motor
doquitok	mretlw	0		;
;
; Subroutine used to stop both motors.
;	w = 1 means kill the power too
;
stopboth
		movwf	temp
		bsf	rstatus, ST_STOP ; Stop RA motion
		bsf	dstatus, ST_STOP ; Stop Dec motion
stopboth1	btfss	rstatus, ST_STOPPED ; Wait for both axes to stop
		goto	stopboth1	;
		btfss	dstatus, ST_STOPPED ;
		goto	stopboth1	;
		movfw	temp		; Idle?
		skpnz			;
		goto	stopbothok	; Done - both are stopped
		movlw	0		;
		ra_out			; Turn off both motors
		movlw	0		;
		dec_out			;
stopbothok	mreturn			;
;	
; Get commands
;
doget
		incf	FSR, f		; Advance to C, hopefully
		movfw	INDF		;
		sublw	'C'		; Get count?
		skpz			;
		goto	doget1		; Jump if not a C
		mcall	dogetsub	;
		splh
		movd	rapos, d0	; Convert rapos first
		spl0
		mcall	dtoa		;
		movlw	' '		;
		movwf	INDF		; Put in a space
		incf	FSR, f		;
		splh
		movd	dpos, d0	; Now convert dec pos
		spl0
		mcall	dtoa		;
		mcall	dogetwrap	; Wrap it up
		mretlw	0		; Successful completion
doget1
		movfw	INDF		;
		sublw	'B'		; Get battery value?
		skpnz			;
		goto	doget2		;
		mretlw 	1		; Error return if not "C" or "B"
doget2
		mcall	dogetsub	; Set up for return
		mcall	getbatt		; Get the battery voltage
		movpwd	d0		; Put in d0
		mcall	dtoa		; Convert to ascii
		movlw	' '		;
		movwf	INDF		;
		incf	FSR, f		;
		movfw	avgbatt		; Get the average too
		movpwd	d0		;
		mcall	dtoa		; 
		mcall	dogetwrap	;
		mretlw	0		; And done
		
;
; Helper subroutine for doget - sets up cmdbuff for a return value
;
dogetsub
		movlw	cmdbuff		; Point to start of buffer
		movwf	FSR		;
		movlw	0x0D		; CR
		movwf	INDF		;
		incf	FSR, f		;
		movlw	0x0A		; LF
		movwf	INDF		;
		incf	FSR, f		;
		mreturn
;
; Wrapup doget routines
;
dogetwrap
		movlw	0x0D		; CR; LF supplied by docmd()
		movwf	INDF		;
		incf	FSR, F		;
		movlw	0		; Mark the end with a null
		movwf	INDF		;
		movlw	cmdbuff		; Set up to send buff
		movwf	FSR		;
		mcall	buffout		; Send the buffer
		mreturn
;
; Set commands
;	Call with FSR pointing into the buffer at the "S"
;	Returns 0 for success, 1 for error
;	SS<speed>		Sets speed for next Slew command (both axes)
;	SC<m> <count>		Sets the saved count for an axis
;	<speed> is "T" for Tracking speed
;		   "G" for Guide
;		   "L" for Lunar
;		   "S" for Solar
;		   "M" for Medium
;		   "F" for Fast
;
doset
		incf	FSR, f		; Advance to "S" or "C"
		movfw	INDF		;
		sublw	'S'		; Speed?
		skpz			;
		goto	doset1		; Skip if not
; Set speed
		incf	FSR, f		; Advance to speed parameter
		clrf	temp		; Index starts at 0
dosetlp
		call	getspeed	; Get the speed name (F, M, etc)
		andlw	0xff		; Is it zero?
		skpnz			;
		goto	dosetbad	; Error return if end of speedtbl
		subwf	INDF, w		; Same as table entry?
		skpnz			;
		goto	doset2		;
		movlw	4		; Advance index
		addwf	temp, f		;
		goto	dosetlp		; Check next
doset2
		incf	temp, f		; Advance index
		call	getspeed	; Get flag
		movwf	slew		; Set slew
		incf	temp, f		;
		call	getspeed	;
		movwf	speedh		; set speedh
		incf	temp, f
		call	getspeed	;
		movfw	speedl		;
		goto	dosetok		; Successful return
doset1
		movfw	INDF		;
		sublw	'C'		; Count?
		skpz			;
		goto	dosetbad	; Nothing else to set
; Set Count
		incf	FSR, f		; Point to R/D
		movfw	INDF		;
		sublw	'R'		; RA?
		skpz			;
		goto	dosetc1		;
		incf	FSR, f		; Advance past R
		mcall	atod		; Convert RA count to binary
		splh			;
		movd	d0, rapos	; Set count
		spl0			;
		goto	dosetok		;
dosetc1		movfw	INDF		;
		sublw	'D'		; Dec?
		skpz			;
		goto	dosetbad	; Error if neither R nor D
		incf	FSR, f		; Advance past D
		mcall	atod		; Convert DEC count to binary
		splh			;
		movd	d0, dpos	; fall through to dosetok
		spl0		
dosetok		mretlw	0		; Good return
dosetbad	mretlw	1		; Bad return
;
;
; Check the limit switches
;	No parameters passed
;	Nothing returned
;	Side effects are somewhat dramatic..
;
chklim

		mreturn
;
; Check the battery voltage
;	No parameters passed
;	Nothing returned
;	Implments a length 16 rolling average.  Anything below 10V
;	shuts things down.
;
chkbatt
		movfw	t0ticks		; Find out if we need to check
		subwf	timbatt, w	; 
		skpnz			;
		goto	chkbattdone	; Done if the same
		incf	timbatt, f	; Follow the timer
		movfw	avgbatt		; Pick up rolling average
		movpwd	d0		; Save in d0
		movlw	15		; Multiply by 15
		mulpwd	d0, d1, temp, temp1, temp2
		mcall	getbatt		; Pick up the current battery value
		addpwd	d0		; Add to it
		clrf	temp		; Assume no rounding
		movfw	d0+b3		; Check for rounding error
		andlw	0x0f		;
		sublw	7		;
		andlw	0x80		;
		skpnz			;
		goto	chkbatt1	;
		movlw	1		; add in 1 to rount it up
		movwf	temp		;
chkbatt1	rrd	d0		; Divide by 16
		rrd	d0		;
		rrd	d0		;
		rrd	d0		;
		movfw	temp		;
		addpwd	d0		; Round if needed
		movfw	d0+b3		; Pick up the new average
		movwf	avgbatt		; And save
		sublw	MINBATT		; Is it still OK?
		andlw	0x80		;
	ifndef SIM
		skpz			;
	endif
		goto	chkbattdone	;
; Shut it down...
		movlw	1		; Want to kill the power
		mcall	stopboth	;
; xxx need to do LED signals...
chkbattdone	mreturn
;
; getbatt gets the value from the battery a-to-d converter
;
getbatt
		movlw	B'11000001'	; FRC, Channel 0, ADON (Turn on converter)
		movwf	ADCON0		;
		movlw	5		; 16 clocks coming up
		movwf	temp		;
getbatt1	decfsz	temp, f		; Spin it down
		goto	getbatt1	;
		bsf	ADCON0, 2	; Set GO/DONE
		nop			; May not need this..
getbatt2	btfsc	ADCON0, 2	; Spin until conversion is done
		goto	getbatt2	;
		movfw	ADRES		; Pick up result
		bcf	ADCON0, 0	; Shut off unit
		mreturn
;
; atod converts ascii to 32bit
; 	Call with fsr pointing at the number
; 	returns the result in d0
;
atod
		movfw	INDF		; Skip past zeros
		sublw	' '		;
		skpz			;
		goto	atod1		; 
		incf	FSR, f		;
		goto	atod		;
atod1		clrf	temp		; temp = 0 means the number is positive
		movfw	INDF		;		
		sublw	'-'		; Is it negative?
		skpz			;
		goto	atod2		;
		incf	temp, f		; temp = 1 means the result is negative
		incf	FSR, f		;
atod2 ; Pointing at the the number, should be all digits from here on
		clrd	d0		; start with zeros
atod3		movlw	'0'		; 
		subwf	INDF, f		; Pick up the number and convert to binary
		movfw	INDF		; Check for less than '0'
		andlw	0x80		;
		skpz			;
		goto	atodend		; Negative means was less than ascii 0
		movfw	INDF		;
		sublw	9		; w = 9 - w
		andlw	0x80		;
		skpz			;
		goto	atodend		; Must have been greater than ascii 9
		movlw	10		; Multiply previous results by 10	
		mulpwd	d0, d1, temp1, temp2, temp3
		movfw	INDF		; Pick up the new digit
		addwd	d0		; Add it in
		incf	FSR, f		; Advance to next char
		goto	atod3		; Check next char
atodend
		movf	temp, f		; Pos?
		skpnz			;
		goto	atodend1	;
		negd	d0, d1, temp1	; Negate if temp is set
atodend1
		mreturn			; Done
;
; dtoa converts 32 bit binary numbers to ascii
; 	Pass start of buffer in FSR.
;	Returns with FSR pointing at the next space past the number
; 	The number should be in d0
;	uses pottbl to decompose the number
;
dtoa
		movfw	d0+b0		; Is it negative?
		andlw	0x80		;
		skpnz			;
		goto	dtoa1		;
		movlw	'-'		; Put in a minus sign
		movwf	INDF		;
		incf	FSR, f		;
		negd	d0, d1, temp1	; Negate d0
dtoa1
		clrf	temp		; Used as an argument to getpot
		movlw	'0'		;
		movwf	INDF		; Start with a zero
dtoa2 ; This loop looks for the first non-zero digit
      ; Surpresses leading zeros
		mcall	nxtpot		;
		subd	d0, d1, temp1	; d0 = d0 - d1
		movfw	d0+b0		;
		andlw	0x80		; Negative?
		skpnz			;
		goto	dtoa3		;
		addd	d0, d1, temp1	; Put it back
		movlw	40		; At the end of pottbl?
		subwf	temp, w		;
		skpz			;
		goto	dtoa2		; Loop to next try
		incf	FSR, f		; Point to next digit
		goto	dtoadone	; It's zero if whole table finished
dtoa3 ; Got a positive result
		movlw	'1'		;
		movwf	INDF		;
dtoa4 ; Main conversion loop
		subd	d0, d1, temp1	;
		movfw	d0+b0		;
		andlw	0x80		;
		skpz			; Continue if still pos
		goto	dtoa5		;
		incf	INDF, f		; Increment the digit
		goto	dtoa4		;
dtoa5 ; d0 went from d0 >= 0 to d0 < 0
		incf	FSR, f		; Advance to next output digit
		movlw	40		; End of table?
		subwf	temp, w		;
		skpnz			;
		goto	dtoadone	; Done if end of table
		movlw	'9'		;
		movwf	INDF		; Start next digit
		mcall	nxtpot		; Get next power of 10
dtoa6 ; on negative side waiting to go positive
		addd	d0, d1, temp1	; Add in power of 10
		movfw	d0+b0		;
		andlw	0x80		;
		skpnz			;
		goto	dtoa7		;
		decf	INDF, f		;
		goto	dtoa6		;
dtoa7 ; d0 went from d0 < 0 to d0 >= 0
		incf	FSR, f		;
		movlw	40		; Are we done yet?
		subwf	temp, w		;
		skpnz			;
		goto	dtoadone	;
		movlw	'0'		;
		movwf	INDF		;
		mcall	nxtpot		;
		goto	dtoa4		;
dtoadone	mreturn			;
;
; nxtpot calls getpot repeatedly and stuffs the 32 bit number
; 	in d1.  Increments temp (in getpot) as a side effect
nxtpot
		call	getpot		; Pick up current power of 10
		movwf	d1+b0		;
		call	getpot		;
		movwf	d1+b1		;
		call	getpot		;
		movwf	d1+b2		;
		call	getpot		;
		movwf	d1+b3		;
		mreturn			;
		END			; directive 'end of program'

